function [ wSig, wAxis ] = wd( s, t )
% [wSig, wAxis] = wd( signal, axis )
%       Computes the 1-dimensional Wigner distribution of the signal 'sig' 
%       with 't' as scale. Some improvements respect previous versions 
%       have been added in order to increase the speed of computation.
%
%       It will return the WD of the signal in the variable 'wSig' and
%       optionally the frequency axis in 'wAxis'. The space axis is the
%       same as used as input argument.
%
%       The Wigner Distribution is a 2D function which is always real, so
%       in the end of this function we study that to check if the numerical
%       calculations are correct. One can change the 'PRECISSION' variable
%       to adjust the difference of magnitude order of the maximum value of
%       the real part and the imaginary part.

    % Variable to allow some feedback from the program
    DEBUG = 0;

    % Difference between the orden of the maximum of the real part and the
    % imaginary part
    PRECISSION = 6;

    % Length of the signal
    N = length(s);

    % Length of the scale axis
    M = length(t);

    % It'd better be the same size
    if ( M ~= N )
        exit;
    end

    % Length (with units) of the signal time axis.
    L = t(N) - t(1);

    % Be sure that there are no zero values and that it has double precision.
    s = double(s+eps);

    % The complex conjugated signal.
    cs = conj(s);

    % Temporal vector for the convolution.
    tmp = zeros(1, N+1);

    % For each value that the product s(t+m)s*(t-m) is not out of bounds do:
    for k=1:N+1
        % Build the left transposition mask.
        left = tmp;
        left(k) = 1;

        % Build the right trasposition mask as the 180-degree rotation of the
        % left transposition mask.
        right = rot90(rot90(left));
        
        % Build the s(t-m) and s(t+m) vectors.
        lA = conv(s, left);
        rA = conv(cs, right);
        A(:,k) = lA .* rA;
    end

    % Compute the Fourier transform of the zero-centered (on the index k)
    % matrix A(:,k). That gives us the Wigner Distribution.
    A = shift1d(A.', 1);
    wSig = shift1d(fft(A,[],1),1)/length(A(:,1));

    % Fit to the right values.
    wSig = wSig(2:N+1,(N/2+1):(3*N/2));

    % The frequency axis.
    wAxis = [-N/2:N/2-1/N]*N/length(A)/L;

    % Check if the maximum value of the real part of wSig is at least
    % PRECISSION orders of magnitud bigger than the imaginary part.
    if( max(max(abs(real(wSig))))/(max(max(abs(imag(wSig))))) < 10^(PRECISSION) )
        fprintf('ERROR: value of imaginary part exceeds limit.\n');
        PRECISSION
    else
        fprintf('INFO: everything seems to have worked as spected.\n');
        wSig = real(wSig);
    end
